local SR_Cooldowns = SpellReminder:NewModule("SR_Cooldowns","AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("SpellReminder")

local function ClipNumber(number, minimum, maximum, trueMax)
	return min(max(min(number,maximum),minimum),trueMax)
end

local defaults = {
	profile = {
		enabled = false,
	},
}
local options = {
	type = "group",
	name = L["Cooldowns"],
	order = 600,
	childGroups = "tab",
	args = {
		enabled = {
			type = "toggle",
			name = L["Enabled"],
			desc = L["Enabled"],
			get = function()
				return SR_Cooldowns:IsEnabled()
			end,
			set = function(info, v)
				if v then
					SR_Cooldowns:Enable()
				else
					SR_Cooldowns:Disable()
				end
				SR_Cooldowns.db.profile.enabled = v
			end,
			order = 100,
		},
	},
}

function SR_Cooldowns:OnInitialize()
	self.db = SpellReminder.db:RegisterNamespace('Cooldowns', defaults)
	db = self.db.profile
	
	if SpellReminder.db.profile.docooldowns then
		self.db.profile.enabled = SpellReminder.db.profile.docooldowns
		SpellReminder.db.profile.docooldowns = nil
	end
	
	self:SetEnabledState(self.db.profile.enabled)
	SpellReminder:RegisterModuleOptions('Cooldowns', options, L["Cooldowns"])
end

function SR_Cooldowns:OnDisable()
	self:UnregisterAllEvents()
	for eventName,info in pairs(SpellReminder.track.active) do 
		if info.tGUID == "COOLDOWN" then
			SpellReminder:CleanUp(eventName)
		end
	end
end

function SR_Cooldowns:OnEnable()
	self:RegisterEvent("SPELL_UPDATE_COOLDOWN","ProcessCooldowns")
	self:RegisterEvent("PLAYER_ENTERING_WORLD","ProcessCooldowns")
	self:ProcessCooldowns()
end

function SR_Cooldowns:ProcessCooldowns()
	local i = 1
	local cooldowns,done = {},{}
	local rangedSpeed = UnitRangedDamage("player")
	
	while true do
		local success, spellName, spellSubname = pcall(GetSpellBookItemName, i, BOOKTYPE_SPELL)
		if (not success) then break end
		if (not spellName) then	break end
		local start, duration, enabled = GetSpellCooldown(i,BOOKTYPE_SPELL);
		if (not done[spellName]) and enabled == 1 and duration > 1.5 and duration > rangedSpeed then
			done[spellName] = true
			local tabName, tabTexture
			local useTexture = GetSpellTexture(i,BOOKTYPE_SPELL)
			
			local remaining = start + duration - GetTime()
			local setupID = 'cd:'..spellName
			local eventName = SpellReminder:GetEventName(setupID,"COOLDOWN")
			
			SpellReminder:SetSpellCategory(setupID,'Cooldowns')-- ('..tabName..')')

			local ci = self:TrackSpell(eventName,setupID,"Cooldown","COOLDOWN",remaining,"cooldown",useTexture)
		else
			-- clear bar
			SpellReminder:CleanUp(eventName)
		end
		i = i + 1
	end
end